class Searching:
    def __init__(self):
        pass
    def perform_binary_search(self, input_array: [], input_element: int):
        left, right = 0, len(input_array) - 1
        while left <= right:
            mid = (left + right)//2
            if input_element == input_array[mid]:
                return mid
            elif input_element < input_array[mid]:
                right = mid - 1
            elif input_element > input_array[mid]:
                left = mid + 1
        return -1


class QueueNode:
    def __init__(self):
        self.data = 0
        self.next = None

class Queue:
    def __init__(self):
        return
    def enqueue(self):
        return
    def dequeue(self):
        return
    def print_queue(self):
        return


class Sorting:

    def __init__(self):
        return

    def perform_selection_sort(self):
        return

    def perform_insertion_sort(self):
        return

    def perform_heap_sort(self):
        return

    def perform_merge_sort(self):
        return

    def perform_quick_sort(self):
        return

    def perform_heap_sort(self):
        return


class LinkedNode:
    def __init__(self):
        self.data = 0
        self.next = None

class LinkedList:
    def __init__(self):
        return
    def append(self):
        return
    def insert_element(self):
        return
    def remove_element(self):
        return
    def print_list(self):
        return


class TrieNode:
    def __init__(self):
        self.nodes = dict()
        self.is_leaf = False


class Trie:
    def __init__(self):
        self.root_node = TrieNode()
        self.__word_list = ""

    """
        def delete(self, word: str):

            def _delete(curr: TrieNode, word: str, index: int):
                if index == len(word):
                    # If word does not exist
                    if not curr.is_leaf:
                        return False
                    curr.is_leaf = False
                    return len(curr.nodes) == 0
                char = word[index]
                char_node = curr.nodes.get(char)
                # If char not in current trie node
                if not char_node:
                    return False
                # Flag to check if node can be deleted
                delete_curr = _delete(char_node, word, index + 1)
                if delete_curr:
                    del curr.nodes[char]
                    return len(curr.nodes) == 0
                return delete_curr

        _delete(self, word, 0)
    """

    def insert_many(self, words: [str]):
        for word in words:
            self.insert(word)

    def insert(self, word: str):
        curr = self.root_node
        for char in word:
            if char not in curr.nodes:
                curr.nodes[char] = TrieNode()
            curr = curr.nodes[char]
        curr.is_leaf = True

    def find(self, word: str):
        curr = self.root_node
        for char in word:
            if char not in curr.nodes:
                return False
            curr = curr.nodes[char]
        return curr.is_leaf

    def __collect_words(self, curr_node: TrieNode, word: str):

        if curr_node.is_leaf:
            self.__word_list = self.__word_list + word + " "

        for key, value in curr_node.nodes.items():
            self.__collect_words(value, word + key)

    def return_words(self):
        self.__collect_words(self.root_node, "")
        return self.__word_list



class StackNode:
    def __init__(self):
        self.data = 0
        self.top = None

class Stack:
    def __init__(self):
        return
    def push(self):
        return
    def pop(self):
        return
    def print_stack(self):
        return 